#!/bin/bash

if [ $# -lt 1 ]; then
	echo "USAGE: prepare_decoder config_file"
	exit 1
fi

# command line parsing
conf=$1

conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf
out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`

if [ -e "$out_dir/decoder/wip_tuning.txt" ] ; then
  exit
fi

# predefined variables
penalty_interval="-20 20"
default_penalty=-4

# create temporal dicectory 
odir=${out_dir}/decoder
mkdir -p $odir

# check files
if [ ! -e "$out_dir/dicts/dict" ] ; then 
	echo "The dictionary '$out_dir/dicts/dict' does not exist";
	exit
fi
if [ ! -e "$out_dir/mlfs/ref.mlf" ] ; then
  echo "The reference MLF '$out_dir/mlfs/ref.mlf' does not exist"
  exit
fi
if [ ! -e "$tune_fea_list" ] ; then
  echo "The crossvalidation list '$tune_fea_list' does not exist"
  exit
fi

# list of posterior files for phoneme insertion penalty tuning
mkdir -p $out_dir/lists
awk -F '=' '{gsub(".*/", "", $1); print "'$out_dir'/posteriors/" $1}' $tune_fea_list  > $out_dir/lists/tune_post.scp

# create a new dictionary and a new phoneme MLF if the actual dictionary contains states
nstates=`cat $out_dir/dicts/dict | grep "__2" | wc -l`
if [ $nstates == 0 ] ; then
  phoneme_dict="$odir/dict"
  ln -s $out_dir/dicts/dict $phoneme_dict
  phoneme_mlf="$odir/ref.mlf"
  ln -s $out_dir/mlfs/ref.mlf $phoneme_mlf
else
  phoneme_dict="$odir/dict"
  cat $out_dir/dicts/dict | sed 's/\(.*\)__[0-9]*$/\1/' | uniq > $phoneme_dict
  phoneme_mlf="$odir/ref.mlf"
  ${script_dir}/misc/mlf_st2phn.pl $out_dir/mlfs/ref.mlf > $phoneme_mlf
fi

# create state list
# it looks like:  phn1 phn1
#                 phn2 phn2
#              or
#                 phn1 phn1_st1 phn1_st2 phn1_st3
#                 phn2 phn2_st1 phn2_st2 phn2_st3
state_list="$odir/state_list"
awk 'BEGIN {prev_phn=""; str="";}
           {
             split($0, a, /__[0-9]/); phn = a[1];
             if(phn != prev_phn && prev_phn != "")
             {
               print prev_phn "\t" str;
               str = "";
             }
             str = str " " $0;
             prev_phn = phn;
           }
     END   { print prev_phn "\t" str;}' $out_dir/dicts/dict > $state_list

# lexicon
lexicon="$odir/lexicon"
awk '{print $1, $1;}' $state_list > $lexicon

# create language nets
cat $phoneme_dict | tr '\n' '|' | awk '{gsub("\\|$", ""); print "(<" $1 ">)";}' > $odir/phn_loop.net
${bin_dir}/HParse $odir/phn_loop.net $odir/phn_loop.lat
 
# create models for SVite
awk '{print $1, NF - 1;}' $state_list > $odir/nstates.txt
${script_dir}/misc/create_like_hmms.pl $odir/nstates.txt > $odir/hmmdefs

# tuning the phoneme insertion penalty
echo "Tuning phoneme insertion penalty ..."
echo "  List:                  $tune_fea_list"
if [ $tune_max_acc == "1" ] ; then
  echo "  Criteria:              maximum accuracy"
  ${script_dir}/misc/tunepenalty_acc.pl $penalty_interval $odir $out_dir/lists/tune_post.scp $phoneme_mlf $phoneme_dict $lexicon $odir/phn_loop.lat 1 $bin_dir | tee $odir/wip_tuning.txt
else
  echo "  Criteria:              equal insertions and deletions"
  ${script_dir}/misc/tunepenalty.pl $penalty_interval $odir $out_dir/lists/tune_post.scp $phoneme_mlf $phoneme_dict $lexicon $odir/phn_loop.lat 1 $bin_dir | tee $odir/wip_tuning.txt
fi

#penalty=`tail -n 1 $odir/wip_tuning.txt`




