#!/bin/bash

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

if [ -e "$conf_global" ] ; then
  source $conf_global
fi
if [ -e "$conf" ] ; then
  source $conf
fi

out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`

# default configuration
target_model_dir="$$"
source_mmf=""
script=""
cv_script=""
lr_estim_script=""
learning_rate=0.08
ts_min_iter=1
ts_max_iter=99999
ts_delta_stop=.5
ts_delta_dec=.5
ts_keep_dec=1

# command line parsing - remove variables used by this script, keep the SNet variables
args="$*"
args=${args#* }       # omit first argument = configuration file
filtered_args=""
for v in `echo $args` ; do
 echo $v
 if [[ "$v" =~ "--target-model-dir=" ]] ; then
   target_model_dir="${v#*=}"
 elif [[ "$v" =~ "--train-dir" ]] ; then
   train_dir="${v#*=}"
 elif [[ "$v" =~ "--source-mmf" ]] ; then
   source_mmf="$source_mmf ${v#*=}"
 elif [[ "$v" =~ "--cv-script" ]] ; then
   cv_script="${v#*=}"
   if [ ! -e "$cv_script" ] ; then
     echo "ERROR: The crossvalidation list '$cv_script' does not exist" > /dev/stderr
     exit 1
   fi
   if [ `wc $cv_script | awk '{print $1}'` == 0 ] ; then
     echo "ERROR: Empty crossvalidation list '$cv_script'" > /dev/stderr
     exit 1
   fi
 elif [[ "$v" =~ "--script" ]] ; then
   script="${v#*=}"
   if [ ! -e "$script" ] ; then
     echo "ERROR: The training list '$script' does not exist" > /dev/stderr
     exit 1
   fi
   if [ `wc $script | awk '{print $1}'` == 0 ] ; then
     echo "ERROR: Empty training list '$script'" > /dev/stderr
     exit 1
   fi   
 elif [[ "$v" =~ "--lr-estim-script" ]] ; then
   lr_estim_script="${v#*=}"
   if [ ! -e "$lr_estim_script" ] ; then
     echo "ERROR: The learning rate estimation list '$lr_estim_script' does not exist" > /dev/stderr
     exit 1
   fi
   if [ `wc $lr_estim_script | awk '{print $1}'` == 0 ] ; then
     echo "ERROR: Empty learning rate estimation list '$lr_estim_script'" > /dev/stderr
     exit 1
   fi      
 elif [[ "$v" =~ "--learning-rate" ]] ; then
   learning_rate="${v#*=}"
   learning_rate=${learning_rate/,/ }
 elif [[ "$v" =~ "--ts-min-iter" ]] ; then
   ts_min_iter="${v#*=}"
 elif [[ "$v" =~ "--ts-max-iter" ]] ; then
   ts_max_iter="${v#*=}"
 elif [[ "$v" =~ "--ts-delta-stop" ]] ; then
   ts_delta_stop="${v#*=}"
 elif [[ "$v" =~ "--ts-delta-dec" ]] ; then
   ts_delta_dec="${v#*=}"
 elif [[ "$v" =~ "--ts-keep-dec" ]] ; then
   ts_keep_dec="${v#*=}"   
 elif [[ "$v" =~ "--crossvalidation" ]] ; then
   echo -n
 else
   filtered_args="$filtered_args $v" 
 fi
done

echo "target_model_dir: $target_model_dir"
echo "train_dir:        $train_dir"
echo "source_mmf:       $source_mmf"
echo "script:           $script"
echo "cv_script:        $cv_script"
echo "lr_estim_script:  $lr_estim_script"
echo "learning_rate:    $learning_rate"
echo "ts_min_iter:      $ts_min_iter"
echo "ts_max_iter:      $ts_max_iter"
echo "ts_delta_stop:    $ts_delta_stop"
echo "ts_delta_dec:     $ts_delta_dec"
echo "ts_keep_dec:      $ts_keep_dec"

# create initial directory for weigths and copy all model files there
mkdir -p $train_dir/0
cp $source_mmf $train_dir/0

# remove paths from all model files
short_source_mmf=""
for f in `echo $source_mmf` ; do
  short_source_mmf="$short_source_mmf ${f##*/}"
done

# initial parameters
it=1
prev_it=0
best_cv_it=0
best_train_acc=-1000
best_cv_acc=-1000
terminate=0
cv_does_not_grow=0
dec_started=0

lr_list=($learning_rate)            # string of numbers -> array
lr=${lr_list[0]}
n_lr_vals=${#lr_list[@]}

echo "iteration, previous iteration, learning rate, train accuracy (estimated train accuracy), CV accuracy, start time - end time, date" > $train_dir/log.txt	

# loop - training iterations
while [ "$terminate" != "1" ] ; do

	echo "Starting iteration $it, learning rate $lr, weights taken from $prev_it..."
  printf "%3d  %3d  %1.8f" $it $prev_it $lr >> $train_dir/log.txt	
  
  start_time=`date "+%T"`

  inp_model_dir="$train_dir/$prev_it"
  out_model_dir="$train_dir/$it"

  mkdir -p $out_model_dir

  # start training
  full_source_mmf=${short_source_mmf// /,$inp_model_dir\/}  # replace spaces (there is one front of each file name) with colons and paths
  full_source_mmf=${full_source_mmf/,/}                     # remove initial semicolon

  if [ ! -e "$out_model_dir/train.log" ] ; then
    rm -f $out_model_dir/train.log
    computer=`uname -n`
    echo "Computer name: $computer" > $out_model_dir/train.log
    ${bin_dir}/SNet $filtered_args \
                    --source-mmf=$full_source_mmf \
                    --target_model_dir=$out_model_dir \
                    --learning_rate=$lr \
                    --script=$script >> $out_model_dir/train.log 2>&1
    cat $out_model_dir/train.log
  fi

  estim_train_acc=`awk '/TR correct:/ {gsub("%", ""); print $5}' $out_model_dir/train.log`
  echo $estim_train_acc

  if [[ $estim_train_acc = "" ]] ; then
    echo "ERROR: SNet crashed during training." > /dev/stderr
    exit 1
  fi

  # evaluation of accuracy on a set used for learning rate adjusting
  if [[ "$n_lr_vals" = "1" ]] && [[ "$lr_estim_script" != "" ]] ; then
    
    full_target_mmf=${short_source_mmf// /,$out_model_dir\/} 
    full_target_mmf=${full_target_mmf/,/}                    

    if [ ! -e "$out_model_dir/train_fwd.log" ] ; then
      ${bin_dir}/SNet $filtered_args \
                      --learning_rate=$lr \
                      --crossvalidation=TRUE \
                      --source-mmf=$full_target_mmf \
                      --script=$lr_estim_script > $out_model_dir/train_fwd.log 2>&1
      cat $out_model_dir/train_fwd.log
    fi

    train_acc=`awk '/CV correct:/ {gsub("%", ""); print $5}' $out_model_dir/train_fwd.log`
    echo $train_acc
    if [[ $train_acc = "" ]] ; then
      echo "ERROR: SNet crashed during evaluation of accuracy learning rate adjusting." > /dev/stderr
      exit 1
    fi
  else
    train_acc=$estim_train_acc
  fi

  # evaluation of accuracy on a crossvalidation set
  if [[ "$cv_script" != "" ]] ; then
    
    full_target_mmf=${short_source_mmf// /,$out_model_dir\/} 
    full_target_mmf=${full_target_mmf/,/}                    

    if [ ! -e "$out_model_dir/cv_fwd.log" ] ; then
      ${bin_dir}/SNet $filtered_args \
                      --learning_rate=$lr \
                      --crossvalidation=TRUE \
                      --source-mmf=$full_target_mmf \
                      --script=$cv_script > $out_model_dir/cv_fwd.log 2>&1
    fi

    cv_acc=`awk '/CV correct:/ {gsub("%", ""); print $5}' $out_model_dir/cv_fwd.log`
    echo $cv_acc
    if [[ $cv_acc = "" ]] ; then
      echo "ERROR: SNet crashed during evaluation of crossvalidation accuracy." > /dev/stderr
      exit 1
    fi   
  else
    cv_acc=$train_acc
  fi

  end_time=`date "+%T %D"`

  printf "  %3.2f (%3.2f)  %3.2f   %s-%s\n" $train_acc $estim_train_acc $cv_acc "$start_time" "$end_time" >> $train_dir/log.txt	

  # new learning rate
  if [[ "$n_lr_vals" = "1" ]] ; then   # a scheduler is used
    echo $lr $train_acc $prev_train_acc
    new_lr=`echo "" | awk '{if(('$train_acc' - '$best_train_acc' < '$ts_delta_dec') || ('$cv_acc' - '$best_cv_acc' < '$ts_delta_dec') || ('$ts_keep_dec' == 1 && '$dec_started' == 1)) print '$lr' / 2; else print '$lr'}'`
    if [ $lr != $new_lr ] ; then
      dec_started=1
    fi
    lr=$new_lr
  else                                 # a list of learning rates is used
    if [[ "${lr_list[$it]}" != "" ]] ; then
      lr="${lr_list[$it]}"
    fi
  fi
  echo "New learning rate: $lr"
  
  # new initial weights
  improved=`echo "" | awk '{if('$train_acc' > '$best_train_acc') print 1; else print 0}'`
  echo "Training accuracy improved: $improved"
  if [[ "$improved" = "1" ]] ; then
    prev_it=$it
    best_train_acc=$train_acc
  fi
  
  # stipping criteria
  stopped_growing=`echo "" | awk '{if('$cv_acc' - '$best_cv_acc' < '$ts_delta_stop') print 1; else print 0}'`
  echo "Crossvalidation accuracy does not grow: $stopped_growing"
  if [[ "$stopped_growing" = "1" ]] ; then
    if [[ "$cv_does_not_grow" = "0" ]] ; then
      cv_does_not_grow=1
    elif [[ $it -ge $ts_min_iter ]] ; then
      terminate=1
    fi
  else
    cv_does_not_grow=0
  fi    
  if [[ $it -ge $ts_max_iter ]] ; then
    terminate=1
  fi
  
  # new best weights - evaluated on the CV set
  improved=`echo "" | awk '{if('$cv_acc' > '$best_cv_acc') print 1; else print 0}'`
  echo "Crossvalidation accuracy improved: $improved"
  if [[ "$improved" = "1" ]] ; then
    best_cv_it=$it
    best_cv_acc=$cv_acc
  fi  
  
  it=$((it + 1))
done

# log the best results
echo "Best train accuracy '$best_train_acc' obtained in iteration '$prev_it'." | tee -a $train_dir/log.txt
echo "Best CV accuracy '$best_cv_acc' obtained in iteration '$best_cv_it'." | tee -a $train_dir/log.txt
it_1=$(( it - 1 ))
echo "Number of iterations '$it_1'." | tee -a $train_dir/log.txt

# copy the best weights
inp_model_dir="$train_dir/$best_cv_it"
full_source_mmf=${short_source_mmf// / $inp_model_dir\/}
mkdir -p $target_model_dir
cp $full_source_mmf $target_model_dir
